==> rtai-0.5.tgz: rtai-0.5/mups_examples/msg_clock/cmdclk.c <==

/* MODULE CLOCK */

#define MODULE
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <ctype.h>

#include <asm/system.h>
#include <asm/i82489.h>
#include <asm/fixmap.h>
#include <asm/smp.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/bitops.h>
#include <asm/atomic.h>
#include <asm/desc.h>

#include <rtai.h>
#include <rtai_mups.h>
#include <rtai_fifos.h>

#include "clock.h"

static enum {stopped, running} Clockstatus;
static RT_TASK Clock;

void CommandClock_Put(char command)
{
	static RT_TASK *ackn = 0;
	unsigned int put = 'a';
	unsigned int msg;

	if (ackn != &Clock) {
		ackn = rt_rpc(&Clock, put, &msg);
	}
	if ((Clockstatus == running) == (command == 'T')) {
		rt_send(&Clock, (unsigned int)command);
	}
}

void CommandClock_Get(char *command)
{
	static RT_TASK *ackn = 0;
	unsigned int get = 'b';
	unsigned int msg;

	if (ackn != &Clock) {
		ackn = rt_rpc(&Clock, get, &msg);
	}
	rt_receive(&Clock, &msg);
	*command = (char)msg;
}

extern int cpu_used[];

static void CommandClock_task(int t)
{
	unsigned int command;
	char R = 'R';
	int ackn = 0;
	RT_TASK *get = (RT_TASK *)0, *put = (RT_TASK *)0, *task;

	Clockstatus = stopped;
	while (ackn != ('a' + 'b')) {
		task = rt_receive((RT_TASK *)0, &command);
		switch (command) {
			case 'b':
				get = task;
				ackn += command;
				break;
			case 'a':
				put = task;
				ackn += command;
				break;
		}
	}
	rt_return(put, command);
	rt_return(get, command);

	while(1) {
		cpu_used[hard_cpu_id()]++;
		switch (Clockstatus) {
			case stopped:
				rt_receive(put, &command);
				if (command == 'R') {
					Clockstatus = running;
				}
				break;
			case running:
				if (rt_receive_if(put, &command)) {
					if (command == 'T') {
						Clockstatus = stopped;
					}
				} else {
					command = R;
				}
				break;
		}
		rt_send(get, command);
	}
}

int init_module(void)
{
	rt_task_init(&Clock, CommandClock_task, 0, 2000, 0, 0, 0, 0);
	rt_task_resume(&Clock);
	return 0;
}

void cleanup_module(void)
{
	rt_task_delete(&Clock);
}

==> rtai-0.9a.tgz: rtai-0.9a/mups_examples/msg_clock/cmdclk.c <==

/* MODULE CLOCK */

#define MODULE
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <ctype.h>

#include <asm/system.h>
#include <asm/i82489.h>
#include <asm/fixmap.h>
#include <asm/smp.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/bitops.h>
#include <asm/atomic.h>
#include <asm/desc.h>

#include <rtai.h>
#include <rtai_sched.h>
#include <rtai_fifos.h>

#include "clock.h"

static enum {stopped, running} Clockstatus;
static RT_TASK Clock;

void CommandClock_Put(char command)
{
	static RT_TASK *ackn = 0;
	unsigned int put = 'a';
	unsigned int msg;

	if (ackn != &Clock) {
		ackn = rt_rpc(&Clock, put, &msg);
	}
	if ((Clockstatus == running) == (command == 'T')) {
		rt_send(&Clock, (unsigned int)command);
	}
}

void CommandClock_Get(char *command)
{
	static RT_TASK *ackn = 0;
	unsigned int get = 'b';
	unsigned int msg;

	if (ackn != &Clock) {
		ackn = rt_rpc(&Clock, get, &msg);
	}
	rt_receive(&Clock, &msg);
	*command = (char)msg;
}

extern int cpu_used[];

static void CommandClock_task(int t)
{
	unsigned int command;
	char R = 'R';
	int ackn = 0;
	RT_TASK *get = (RT_TASK *)0, *put = (RT_TASK *)0, *task;

	Clockstatus = stopped;
	while (ackn != ('a' + 'b')) {
		task = rt_receive((RT_TASK *)0, &command);
		switch (command) {
			case 'b':
				get = task;
				ackn += command;
				break;
			case 'a':
				put = task;
				ackn += command;
				break;
		}
	}
	rt_return(put, command);
	rt_return(get, command);

	while(1) {
		cpu_used[hard_cpu_id()]++;
		switch (Clockstatus) {
			case stopped:
				rt_receive(put, &command);
				if (command == 'R') {
					Clockstatus = running;
				}
				break;
			case running:
				if (rt_receive_if(put, &command)) {
					if (command == 'T') {
						Clockstatus = stopped;
					}
				} else {
					command = R;
				}
				break;
		}
		rt_send(get, command);
	}
}

int init_module(void)
{
	rt_task_init_cpuid(&Clock, CommandClock_task, 0, 2000, 0, 0, 0, 0);
	rt_task_resume(&Clock);
	return 0;
}

void cleanup_module(void)
{
	rt_task_delete(&Clock);
}
